@@ -26,7 +26,6 @@ const Gd = imports.gi.Gd;
 const Gdk = imports.gi.Gdk;
 const GData = imports.gi.GData;
 const GLib = imports.gi.GLib;
-const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
 const Zpj = imports.gi.Zpj;
 const _ = imports.gettext.gettext;
@@ -37,6 +36,7 @@ const Signals = imports.signals;
 const ChangeMonitor = imports.changeMonitor;
 const Global = imports.global;
 const Manager = imports.manager;
+const Notifications = imports.notifications;
 const Path = imports.path;
 const Query = imports.query;
 const Searchbar = imports.searchbar;
@@ -598,6 +598,21 @@ const DocCommon = new Lang.Class({
         }
     },
 
+    print: function(toplevel) {
+        this.load(null, Lang.bind(this,
+            function(doc, docModel, error) {
+                if (error) {
+                    log('Unable to print document ' + this.uri + ': ' + error);
+                    return;
+                }
+
+                let printOp = EvView.PrintOperation.new(docModel.get_document());
+                let printNotification = new Notifications.PrintNotification(printOp, doc);
+
+                printOp.run(toplevel);
+            }));
+    },
+
     setFavorite: function(favorite) {
         TrackerUtils.setFavorite(this.id, favorite, null);
     },
@@ -609,13 +624,6 @@ const DocCommon = new Lang.Class({
             retval = '{ ?urn nie:isPartOf <' + this.id + '> }';
 
         return retval;
-    },
-
-    _finishLoad: function(docModel, callback, exception) {
-        if (exception)
-            Global.errorHandler.addLoadError(this, exception);
-
-        callback(this, docModel, exception);
     }
 });
 Signals.addSignalMethods(DocCommon.prototype);
@@ -650,9 +658,9 @@ const LocalDocument = new Lang.Class({
             function(source, res) {
                 try {
                     let docModel = Gd.pdf_loader_load_uri_finish(res);
-                    this._finishLoad(docModel, callback, null);
+                    callback(this, docModel, null);
                 } catch (e) {
-                    this._finishLoad(null, callback, e);
+                    callback(this, null, e);
                 }
             }));
     },
@@ -690,7 +698,7 @@ const GoogleDocument = new Lang.Class({
     _createGDataEntry: function(cancellable, callback) {
         let source = Global.sourceManager.getItemById(this.resourceUrn);
 
-        let authorizer = new Gd.GDataGoaAuthorizer({ goa_object: source.object });
+        let authorizer = new GData.GoaAuthorizer({ goa_object: source.object });
         let service = new GData.DocumentsService({ authorizer: authorizer });
 
         service.query_single_entry_async
@@ -721,10 +729,10 @@ const GoogleDocument = new Lang.Class({
                         function(source, res) {
                             try {
                                 let docModel = Gd.pdf_loader_load_uri_finish(res);
-                                this._finishLoad(docModel, callback, null);
+                                callback(this, docModel, null);
                             } catch (e) {
                                 // report the outmost error only
-                                this._finishLoad(null, callback, exception);
+                                callback(this, null, exception);
                             }
                         }));
 
@@ -736,9 +744,9 @@ const GoogleDocument = new Lang.Class({
                         function(source, res) {
                             try {
                                 let docModel = Gd.pdf_loader_load_uri_finish(res);
-                                this._finishLoad(docModel, callback, null);
+                                callback(this, docModel, null);
                             } catch (e) {
-                                this._finishLoad(null, callback, e);
+                                callback(this, null, e);
                             }
                         }));
             }));
@@ -860,10 +868,10 @@ const SkydriveDocument = new Lang.Class({
                         function(source, res) {
                             try {
                                 let docModel = Gd.pdf_loader_load_uri_finish(res);
-                                this._finishLoad(docModel, callback, null);
+                                callback(this, docModel, null);
                             } catch (e) {
                                 // report the outmost error only
-                                this._finishLoad(null, callback, exception);
+                                callback(this, null, exception);
                             }
                         }));
 
@@ -875,9 +883,9 @@ const SkydriveDocument = new Lang.Class({
                         function(source, res) {
                             try {
                                 let docModel = Gd.pdf_loader_load_zpj_entry_finish(res);
-                                this._finishLoad(docModel, callback, null);
+                                callback(this, docModel, null);
                             } catch (e) {
-                                this._finishLoad(null, callback, e);
+                                callback(this, null, e);
                             }
                         }));
             }));
@@ -913,7 +921,9 @@ const DocumentManager = new Lang.Class({
     _init: function() {
         this.parent();
 
-        this._model = new DocumentModel();
+        this._activeDocModel = null;
+        this._activeDocModelIds = [];
+        this._loaderCancellable = null;
 
         Global.changeMonitor.connect('changes-pending',
                                      Lang.bind(this, this._onChangesPending));
@@ -934,8 +944,6 @@ const DocumentManager = new Lang.Class({
                 let doc = this.getItemById(changeEvent.urn);
 
                 if (doc) {
-                    this._model.documentRemoved(doc);
-
                     doc.destroy();
                     this.removeItemById(changeEvent.urn);
 
@@ -984,7 +992,6 @@ const DocumentManager = new Lang.Class({
     addDocumentFromCursor: function(cursor) {
         let doc = this.createDocumentFromCursor(cursor);
         this.addItem(doc);
-        this._model.documentAdded(doc);
 
         if (doc.collection)
             Global.collectionManager.addItem(doc);
@@ -999,13 +1006,38 @@ const DocumentManager = new Lang.Class({
         };
 
         this.parent();
-        this._model.clear();
+    },
+
+    _onDocumentLoadError: function(doc, error) {
+        if (error.matches(Gio.IOErrorEnum, Gio.IOErrorEnum.CANCELLED))
+            return;
+
+        // Translators: %s is the title of a document
+        let message = _("Unable to load \"%s\" for preview").format(doc.name);
+        this.emit('load-error', doc, message, error);
+    },
+
+    _onDocumentLoaded: function(doc, docModel, error) {
+        if (error) {
+            this._onDocumentLoadError(doc, error);
+            return;
+        }
+
+        // save loaded model and signal
+        this._activeDocModel = docModel;
+        this.emit('load-finished', doc, docModel);
+
+        // load metadata
+        this._connectMetadata(docModel);
     },
 
     setActiveItem: function(doc) {
         if (!this.parent(doc))
             return;
 
+        // cleanup any state we have for previously loaded model
+        this._clearActiveDocModel();
+
         if (!doc)
             return;
 
@@ -1016,69 +1048,49 @@ const DocumentManager = new Lang.Class({
 
         let recentManager = Gtk.RecentManager.get_default();
         recentManager.add_item(doc.uri);
+
+        this._loaderCancellable = new Gio.Cancellable();
+        doc.load(this._loaderCancellable, Lang.bind(this, this._onDocumentLoaded));
+        this.emit('load-started', doc);
     },
 
-    getModel: function() {
-        return this._model;
-    }
-});
-
-const DocumentModel = new Lang.Class({
-    Name: 'DocumentModel',
-
-    _init: function() {
-        this.model = Gtk.ListStore.new(
-            [ GObject.TYPE_STRING,
-              GObject.TYPE_STRING,
-              GObject.TYPE_STRING,
-              GObject.TYPE_STRING,
-              GdkPixbuf.Pixbuf,
-              GObject.TYPE_LONG,
-              GObject.TYPE_BOOLEAN ]);
-        this.model.set_sort_column_id(Gd.MainColumns.MTIME,
-                                      Gtk.SortType.DESCENDING);
-    },
-
-    clear: function() {
-        this.model.clear();
+    _clearActiveDocModel: function() {
+        // cancel any pending load operation
+        if (this._loaderCancellable) {
+            this._loaderCancellable.cancel();
+            this._loaderCancellable = null;
+        }
+
+        // clear any previously loaded document model
+        if (this._activeDocModel) {
+            this._activeDocModelIds.forEach(Lang.bind(this,
+                function(id) {
+                    this._activeDocModel.disconnect(id);
+                }));
+
+            this._activeDocModel = null;
+            this._activeDocModelIds = [];
+        }
     },
-
-    documentAdded: function(doc) {
-        let iter = this.model.append();
-        this.model.set(iter,
-            [ 0, 1, 2, 3, 4, 5 ],
-            [ doc.id, doc.uri, doc.name,
-              doc.author, doc.pixbuf, doc.mtime ]);
-
-        let treePath = this.model.get_path(iter);
-        let treeRowRef = Gtk.TreeRowReference.new(this.model, treePath);
-
-        doc.connect('info-updated', Lang.bind(this,
-            function() {
-                let objectPath = treeRowRef.get_path();
-                if (!objectPath)
-                    return;
-
-                let objectIter = this.model.get_iter(objectPath)[1];
-                if (objectIter)
-                    this.model.set(objectIter,
-                        [ 0, 1, 2, 3, 4, 5 ],
-                        [ doc.id, doc.uri, doc.name,
-                          doc.author, doc.pixbuf, doc.mtime ]);
-            }));
-    },
-
-    documentRemoved: function(doc) {
-        this.model.foreach(Lang.bind(this,
-            function(model, path, iter) {
-                let id = model.get_value(iter, Gd.MainColumns.ID);
-
-                if (id == doc.id) {
-                    this.model.remove(iter);
-                    return true;
-                }
 
-                return false;
-            }));
+    _connectMetadata: function(docModel) {
+        let evDoc = docModel.get_document();
+        let file = Gio.File.new_for_uri(evDoc.get_uri());
+
+        if (!Gd.is_metadata_supported_for_file(file))
+            return;
+
+        let metadata = new Gd.Metadata({ file: file });
+
+        // save current page in metadata
+        let [res, val] = metadata.get_int('page');
+        if (res)
+            docModel.set_page(val);
+        this._activeDocModelIds.push(
+            docModel.connect('page-changed', Lang.bind(this,
+                function(source, oldPage, newPage) {
+                    metadata.set_int('page', newPage);
+                }))
+        );
     }
 });
