@@ -46,7 +46,6 @@ const Embed = new Lang.Class({
     Name: 'Embed',
 
     _init: function() {
-        this._loaderCancellable = null;
         this._queryErrorId = 0;
 
         this.widget = new GtkClutter.Embed({ use_layout_size: true });
@@ -66,11 +65,6 @@ const Embed = new Lang.Class({
         this._overlayLayout.add(this._contentsActor,
             Clutter.BinAlignment.FILL, Clutter.BinAlignment.FILL);
 
-        // pack the toolbar
-        this._toolbar = new MainToolbar.OverviewToolbar();
-        this._contentsActor.add_actor(this._toolbar.actor);
-        this._contentsLayout.set_fill(this._toolbar.actor, true, false);
-
         // pack the main GtkNotebook and a spinnerbox in a BinLayout, so that
         // we can easily bring them front/back
         this._viewLayout = new Clutter.BinLayout();
@@ -103,13 +97,6 @@ const Embed = new Lang.Class({
             Clutter.BinAlignment.FILL, Clutter.BinAlignment.FILL);
         this._background.lower_bottom();
 
-        // create the dropdown for the search bar, it's hidden by default
-        this._dropdownBox = new Searchbar.Dropdown();
-        this._overlayLayout.add(this._dropdownBox.actor,
-            Clutter.BinAlignment.CENTER, Clutter.BinAlignment.FIXED);
-        this._dropdownBox.actor.add_constraint(new Clutter.BindConstraint({ source: this._toolbar.toolbarActor,
-                                                                            coordinate: Clutter.BindCoordinate.Y }));
-
         // create the OSD toolbar for selected items, it's hidden by default
         this._selectionToolbar = new Selections.SelectionToolbar(this._contentsActor);
         this._overlayLayout.add(this._selectionToolbar.actor,
@@ -126,17 +113,23 @@ const Embed = new Lang.Class({
         this._preview = new Preview.PreviewView();
         this._previewPage = this._notebook.append_page(this._preview.widget, null);
 
-        Global.errorHandler.connect('load-error',
-                                    Lang.bind(this, this._onLoadError));
-
         Global.modeController.connect('window-mode-changed',
                                       Lang.bind(this, this._onWindowModeChanged));
         Global.modeController.connect('fullscreen-changed',
                                       Lang.bind(this, this._onFullscreenChanged));
         Global.trackerController.connect('query-status-changed',
                                          Lang.bind(this, this._onQueryStatusChanged));
+        Global.trackerController.connect('query-error',
+                                         Lang.bind(this, this._onQueryError));
+
         Global.documentManager.connect('active-changed',
                                        Lang.bind(this, this._onActiveItemChanged));
+        Global.documentManager.connect('load-started',
+                                       Lang.bind(this, this._onLoadStarted));
+        Global.documentManager.connect('load-finished',
+                                       Lang.bind(this, this._onLoadFinished));
+        Global.documentManager.connect('load-error',
+                                       Lang.bind(this, this._onLoadError));
 
         this._onQueryStatusChanged();
     },
@@ -152,6 +145,10 @@ const Embed = new Lang.Class({
         }
     },
 
+    _onQueryError: function(manager, message, exception) {
+        this._setError(message, exception.message);
+    },
+
     _onFullscreenChanged: function(controller, fullscreen) {
         if (fullscreen) {
             this._previewFullscreen = new Preview.PreviewFullscreen(this._preview, this._overlayLayout, this._contentsActor);
@@ -191,34 +188,25 @@ const Embed = new Lang.Class({
             this._windowModeChangeFlash();
     },
 
-    _onActiveItemChanged: function() {
-        let doc = Global.documentManager.getActiveItem();
+    _onActiveItemChanged: function(manager, doc) {
+        let newMode = WindowMode.WindowMode.OVERVIEW;
 
-        if (!doc)
-            return;
-
-        let collection = Global.collectionManager.getItemById(doc.id);
-        if (collection) {
-            Global.modeController.setWindowMode(WindowMode.WindowMode.OVERVIEW);
-            return;
+        if (doc) {
+            let collection = Global.collectionManager.getItemById(doc.id);
+            if (!collection)
+                newMode = WindowMode.WindowMode.PREVIEW;
         }
 
-        // switch to preview mode, and schedule the spinnerbox to
-        // move in if the document is not loaded by the timeout
-        Global.modeController.setWindowMode(WindowMode.WindowMode.PREVIEW);
-        this._spinnerBox.moveInDelayed(_PDF_LOADER_TIMEOUT);
-
-        this._loaderCancellable = new Gio.Cancellable();
-        doc.load(this._loaderCancellable, Lang.bind(this, this._onDocumentLoaded));
+        Global.modeController.setWindowMode(newMode);
     },
 
-    _onDocumentLoaded: function(doc, docModel, error) {
-        this._loaderCancellable = null;
-
-        if (!docModel) {
-            return;
-        }
+    _onLoadStarted: function() {
+        // switch to preview mode, and schedule the spinnerbox to
+        // move in if the document is not loaded by the timeout
+        this._spinnerBox.moveInDelayed(_PDF_LOADER_TIMEOUT);
+    },
 
+    _onLoadFinished: function(manager, doc, docModel) {
         this._toolbar.setModel(docModel);
         this._preview.setModel(docModel);
         this._preview.widget.grab_focus();
@@ -227,34 +215,39 @@ const Embed = new Lang.Class({
         Global.modeController.setCanFullscreen(true);
     },
 
+    _onLoadError: function(manager, doc, message, exception) {
+        this._spinnerBox.moveOut();
+        this._setError(message, exception.message);
+    },
+
     _prepareForOverview: function() {
         if (this._preview)
             this._preview.setModel(null);
 
-        if (this._loaderCancellable) {
-            this._loaderCancellable.cancel();
-            this._loaderCancellable = null;
-        }
+        if (this._toolbar)
+            this._toolbar.actor.destroy();
+
+        // pack the toolbar
+        this._toolbar = new MainToolbar.OverviewToolbar(this._viewLayout);
+        this._contentsLayout.pack_start = true;
+        this._contentsActor.add_actor(this._toolbar.actor);
+        this._contentsLayout.set_fill(this._toolbar.actor, true, false);
 
         this._spinnerBox.moveOut();
         this._errorBox.moveOut();
 
-        this._queryErrorId =
-            Global.errorHandler.connect('query-error',
-                                        Lang.bind(this, this._onQueryError));
-
         this._notebook.set_current_page(this._viewPage);
     },
 
-    _onQueryError: function(manager, message, exception) {
-        this._setError(message, exception.message);
-    },
-
     _prepareForPreview: function() {
-        if (this._queryErrorId != 0) {
-            Global.errorHandler.disconnect(this._queryErrorId);
-            this._queryErrorId = 0;
-        }
+        if (this._toolbar)
+            this._toolbar.actor.destroy();
+
+        // pack the toolbar
+        this._toolbar = new Preview.PreviewToolbar(this._preview);
+        this._contentsLayout.pack_start = true;
+        this._contentsActor.add_actor(this._toolbar.actor);
+        this._contentsLayout.set_fill(this._toolbar.actor, true, false);
 
         this._notebook.set_current_page(this._previewPage);
     },
@@ -264,10 +257,7 @@ const Embed = new Lang.Class({
         this._errorBox.moveIn();
     },
 
-    _onLoadError: function(manager, message, exception) {
-        this._loaderCancellable = null;
-        this._spinnerBox.moveOut();
-
-        this._setError(message, exception.message);
+    getMainToolbar: function(event) {
+        return this._toolbar;
     }
 });
