@@ -111,7 +111,7 @@ const QueryBuilder = new Lang.Class({
         return sparql;
     },
 
-    _buildFilterString: function() {
+    _buildFilterString: function(currentType) {
         let sparql = 'FILTER (';
 
         sparql += Global.searchMatchManager.getFilter();
@@ -119,8 +119,11 @@ const QueryBuilder = new Lang.Class({
         sparql += Global.sourceManager.getFilter();
         sparql += ' && ';
         sparql += Global.searchCategoryManager.getFilter();
-        sparql += ' && ';
-        sparql += Global.searchTypeManager.getFilter();
+
+        if (currentType) {
+            sparql += ' && ';
+            sparql += currentType.getFilter();
+        }
 
         sparql += ')';
 
@@ -135,28 +138,51 @@ const QueryBuilder = new Lang.Class({
         return sparql;
     },
 
+    _buildWhere: function(global, flags) {
+        let whereSparql = 'WHERE { ';
+        let whereParts = [];
+        let searchTypes = [];
+
+        if (flags & QueryFlags.UNFILTERED)
+            searchTypes = Global.searchTypeManager.getAllTypes();
+        else
+            searchTypes = Global.searchTypeManager.getCurrentTypes();
+
+        // build an array of WHERE clauses; each clause maps to one
+        // type of resource we're looking for.
+        searchTypes.forEach(Lang.bind(this,
+            function(currentType) {
+                let part = '{ ' + currentType.getWhere() + this._buildOptional();
+
+                if ((flags & QueryFlags.UNFILTERED) == 0) {
+                    if (global)
+                        part += Global.searchCategoryManager.getWhere() +
+                                Global.collectionManager.getWhere();
+
+                    part += this._buildFilterString(currentType);
+                }
+
+                part += ' }';
+                whereParts.push(part);
+            }));
+
+        // put all the clauses in an UNION
+        whereSparql += whereParts.join(' UNION ');
+        whereSparql += ' }';
+
+        return whereSparql;
+    },
+
     _buildQueryInternal: function(global, flags) {
-        let globalSparql =
-            'WHERE { ?urn a rdfs:Resource ' +
-            this._buildOptional();
+        let whereSparql = this._buildWhere(global, flags);
+        let tailSparql = '';
 
+        // order results by mtime
         if (global) {
-            if ((flags & QueryFlags.UNFILTERED) == 0)
-                globalSparql +=
-                    Global.searchCategoryManager.getWhere() +
-                    Global.collectionManager.getWhere() +
-                    this._buildFilterString();
-
-            globalSparql +=
-                ' } ' +
+            tailSparql +=
                 'ORDER BY DESC (?mtime)' +
                 ('LIMIT %d OFFSET %d').format(Global.offsetController.getOffsetStep(),
                                               Global.offsetController.getOffset());
-        } else {
-            if ((flags & QueryFlags.UNFILTERED) == 0)
-                globalSparql += this._buildFilterString();
-
-            globalSparql += ' }';
         }
 
         let sparql =
@@ -172,18 +198,12 @@ const QueryBuilder = new Lang.Class({
             'nie:dataSource(?urn) ' + // resource URN
             '( EXISTS { ?urn nao:hasTag nao:predefined-tag-favorite } ) ' + // favorite
             '( EXISTS { ?urn nco:contributor ?contributor FILTER ( ?contributor != ?creator ) } ) ' + // shared
-            globalSparql;
+            whereSparql + tailSparql;
 
         return sparql;
     },
 
-    buildSingleQuery: function() {
-        let resource = arguments[0];
-        let flags = QueryFlags.NONE;
-
-        if (arguments.length == 2)
-            flags = arguments[1];
-
+    buildSingleQuery: function(flags, resource) {
         let sparql = this._buildQueryInternal(false, flags);
         sparql = sparql.replace('?urn', '<' + resource + '>', 'g');
 
@@ -195,11 +215,8 @@ const QueryBuilder = new Lang.Class({
     },
 
     buildCountQuery: function() {
-        let sparql =
-            'SELECT DISTINCT COUNT(?urn) WHERE { ' +
-            this._buildOptional() +
-            this._buildFilterString() +
-            '}';
+        let sparql = 'SELECT DISTINCT COUNT(?urn) ' +
+            this._buildWhere(true, QueryFlags.NONE);
 
         return new Query(sparql);
     },
